set define off

create or replace package body Library_Data is

cursor	driver is
select	a.abstract_id, a.title, a.description, a.classification_code,
	a.sub_class_code, a.person_id, a.toc_page_num, a.total_pages,
	sa.paper_id
from	sessions		s,
	abstract_sessions	sa,
	abstracts		a
where	'Yes'			= common.paper_OK_to_publish(a.abstract_id)
and	sa.abstract_id		= a.abstract_id
and	s.session_id		= sa.session_id
order by
	s.sort_order, s.session_id, sa.session_seq;

type	abs_recs is table of driver%ROWTYPE index by binary_integer;

-- **************************************************************************************
-- **************************************************************************************

function OK_to_generate (filename varchar2) return boolean is

begin

   if common.JACoW_URL$ is null then
      common.public_header ('Library Data');
      htp.header (3, 'Paper URL to JACoW Publications is missing.');
      htp.p ('Please set this value in the Post Conference section of the System Parameters area.');
      common.footer;
      return false;
   end if;

   owa_util.mime_header ('application/vnd.wordpad', false, common.charset$);
   htp.p ('Content-Disposition: filename="' || replace (common.conf_title$, ' ', '_') || '_' || filename ||'.txt"');
   owa_util.http_header_close;
   return true;

end OK_to_generate;

-- **************************************************************************************

function load_abstracts return abs_recs is

recs	abs_recs;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   return recs;

end load_abstracts;

-- **************************************************************************************

procedure write (buffer varchar2, new_line boolean default true) is

begin

   htp.prn (buffer);

   if new_line then
      htp.prn (common.crlf$);
   end if;

end write;

-- ***************************************************************************************

function fmt (buffer varchar2) return varchar2 is

str1$	constant varchar2(2)	:= ' ,';
str2$	constant varchar2(1)	:= ',';
str3$	constant varchar2(2)	:= '),';
str4$	constant varchar2(2)	:= ');';

begin

   return replace (replace (buffer, str1$, str2$), str3$, str4$);

end fmt;

-- ***************************************************************************************
-- ***************************************************************************************

procedure BibTex_authors (abs_id integer) is

cursor	driver (acode$ varchar2) is
select	distinct
	p.sort_lname, p.sort_fname, p.sort_mname,
	p.person_id, p.lname, p.fname, p.mname
from	persons		p,
	profiles	prof,
	authors		a
where	a.abstract_id	= abs_id
and	a.author_code	= acode$
and	prof.profile_id	= a.profile_id
and	p.person_id	= prof.person_id
order by p.sort_lname, p.sort_fname, p.sort_mname;

cursor	C2 is
select	common.get_affil_name (addr.affiliation_id, ' ', 'N') affil_name
from	addresses		addr,
	profiles		prof,
	authors			auth
where	auth.abstract_id	= abs_id
and	auth.author_code	= common.primary_author$
and	prof.profile_id		= auth.profile_id
and	addr.address_id		= prof.address_id;

type		rec_type is table of driver%ROWTYPE index by binary_integer;
recs		rec_type;
prim_rec	driver%ROWTYPE;

C2_rec		C2%ROWTYPE;

sep_str		varchar2(5)	:= null;

begin

-- Get the primary author
   open driver (common.primary_author$);
   fetch driver into prim_rec;
   close driver;

-- Get the coauthors
   open driver (common.co_author$);
   fetch driver BULK COLLECT into recs;
   close driver;

   if recs.COUNT > 0 then
      recs(recs.COUNT+1) := null;

-- Move the coauthors down in the array elements by one slot
      for i in reverse 2..recs.COUNT LOOP
         recs(i) := recs(i-1);
      end loop;
   end if;

-- Put the primary author in slot one
   recs(1) := prim_rec;

-- Write the author records   
   write ('  author = {', false);

   for i in 1..recs.COUNT LOOP
      if i = 1 or recs(i).person_id <> prim_rec.person_id then
         write (sep_str || recs(i).lname || ', ' || recs(i).fname, false);
         if recs(i).mname is not null then
            write (' ' || recs(i).mname, false);
         end if;
      end if;

      sep_str := ' and ';
   end loop;

   open C2;
   fetch C2 into C2_rec;
   close C2;

   write ('},');
   write ('  address {' || C2_rec.affil_name || '},');

end BibTex_authors;

-- ***************************************************************************************

procedure BibTex is

recs	abs_recs;
end_pg	integer;

begin

   if OK_to_generate ('BibTex') then
      write ('@PROCEEDINGS {conference:' || common.conf_title$ || ',');
      write ('  title = {Proceedings of the ' || common.conf_title$ || ' Conference},');
      write ('  year = {' || to_char (common.conf_start_monday$, 'YYYY') || '},');
      write ('  month = {' || to_char (common.conf_start_monday$, 'FMMonth') || '},');
      write ('  publisher = {Joint Accelerator Conferences Website},');
      write ('  URL = {http://www.jacow.org}');
      write ('}');

      recs := load_abstracts;
      for i in 1..recs.COUNT LOOP
         end_pg := recs(i).toc_page_num + recs(i).total_pages-1;

         write ('@INPROCEEDINGS {author:' || recs(i).paper_id || ',');
         write ('  title = {' || recs(i).title || '},');
         write ('  pages = {' || recs(i).toc_page_num || '--' ||
			      trim (to_char (end_pg)) || '},');
         BibTeX_authors (recs(i).abstract_id);
         write ('  URL = {' || nvl (common.JACoW_URL$, '???') || upper (recs(i).paper_id) || '.PDF},');
         write ('  crossref = {conference:' || common.conf_title$ || '}');
      end loop;
   end if;

end BibTex;

-- ***************************************************************************************
-- ***************************************************************************************

procedure spires_authors (this_id integer) is

cursor	driver is
select	lname || ', ' || iname			name,
	common.get_affil_by_abs_person (abstract_id, person_id, 'N')	affiliation
from	abstract_authors
where	abstract_id	= this_id
and	ctype		in ('Primary Author', 'Co-Author')
order by
	2, 1;

prev_affil	varchar2(4000) := '~';

begin

   for driver_rec in driver LOOP

      if prev_affil <> driver_rec.affiliation then
         if prev_affil <> '~' then
            write ('  AFFILIATION = ' || prev_affil || ';');
         end if;

         write (' ASTR;');
      end if;

      prev_affil := driver_rec.affiliation;
      write ('  AUTHOR = ' || driver_rec.name || ';');

   end loop;

   write ('  AFFILIATION = ' || prev_affil || ';');

end spires_authors;

-- ***************************************************************************************

procedure spires is

date_str	varchar2(10) := to_char (common.conf_start_monday$, 'Mon YYYY');
page1		integer;
page2		integer;
recs		abs_recs;

begin


   if OK_to_generate ('SPIRES') then
      recs := load_abstracts;

      for i in 1..recs.COUNT LOOP
         page1 := recs(i).toc_page_num;
         page2 := page1 + recs(i).total_pages - 1;

         write ('add;');
         write (' DOC-TYPE = Conference Paper;');
         write (' REPORT-NUM = ' || common.conf_title$ || '-' || recs(i).paper_id || ';');

         spires_authors (recs(i).abstract_id);

         write (' TITLE = ' || recs(i).title || ';');
         write (' DATE = ' || date_str || ';');
         write (' PPF-SUBJECT = I;');
         write (' P = ' || page1 || ' - ' || page2 || ';');
         write (' MEETING-NOTE = ' || common.spires_meeting_note$ || ';');
         write (' TT = ' || recs(i).title || ';');
         write (' CONF-NUMBER = ' || common.spires_conf_number$ || ';');
         write (' URL = ' || common.JACoW_URL$ || recs(i).paper_id || '.PDF;');
         write (';');
      end loop;
   end if;

end spires;

-- ***************************************************************************************

procedure spires2 is

recs		abs_recs;

begin

   if OK_to_generate ('SPIRES2') then
      recs := load_abstracts;

      for i in 1..recs.COUNT LOOP
         write ('addupd;');
         write (' paper =' || common.conf_title$ || '-' || recs(i).paper_id || ';');
         write (' abstract =' || recs(i).description || ';');
         write (';');
      end loop;
   end if;

end spires2;

-- **********************************************************************************************
-- **********************************************************************************************

procedure keywords (aid integer) is

cursor	driver is
select	keyword_descr
from	keyword_codes		k,
	abstract_keywords	ak
where	ak.abstract_id		= aid
and	k.keyword_code		= ak.keyword_code
order by
	keyword_descr;

keywords$	constant varchar2(4)	:= 'KW: ';
sep_str		varchar2(4)		:= keywords$;

begin

   for driver_rec in driver LOOP
      write (sep_str || driver_rec.keyword_descr, false);
      sep_str := ', ';
   end loop;
   write (null);

end keywords;

-- **********************************************************************************************

procedure OAI is

class_descr	classification_codes.classification_descr%TYPE;
sclass_descr	sub_class_codes.sub_class_descr%TYPE;

title$		constant varchar2(4)	:= 'TI: ';
authors$	constant varchar2(4)	:= 'AU: ';
class$		constant varchar2(4)	:= 'SE: ';
sclass$		constant varchar2(4)	:= 'CL: ';
pages$		constant varchar2(4)	:= 'NP: ';
toc$		constant varchar2(4)	:= 'PP: ';
email$		constant varchar2(4)	:= 'EM: ';
URL$		constant varchar2(200)	:= 'URL: ' || common.JACoW_URL$;

email		profiles.email_address%TYPE;
profile_id	integer;
recs		abs_recs;

begin

   if OK_to_generate ('OAI') then
      recs := load_abstracts;
      for i in 1..recs.COUNT LOOP
         class_descr	:= common.decode_value ('classification_descr', 'classification_codes',
						'classification_code', recs(i).classification_code);
         sclass_descr	:= common.decode_value ('sub_class_descr', 'sub_class_codes',
						'sub_class_code', recs(i).sub_class_code);

         profile_id	:= common.get_author_profile_id (recs(i).abstract_id, common.speaker$);
         email		:= common.decode_value ('email_address', 'profiles',
						'profile_id', profile_id);

         write (title$	|| recs(i).title);
         write (authors$	|| fmt (common.get_author_list2 (recs(i).abstract_id)) );
         keywords (recs(i).abstract_id);
         write (class$	|| class_descr);
         write (sclass$	|| sclass_descr);
         write (pages$	|| recs(i).total_pages);
         write (toc$	|| recs(i).toc_page_num);
         write (email$	|| email);
         write (URL$	|| common.JACoW_URL$ || recs(i).paper_id);
      end loop;
   end if;

end OAI;

-- ***************************************************************************************
-- ***************************************************************************************

end Library_Data;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
