set define off

CREATE OR REPLACE package body program as

rpt_title$	constant varchar2(20)	:= 'Program';

type		time_type is record (
			beg_time	number,
			end_time	number);
time_rec	time_type;


type		col_rec is record (
			day	date,
			lcode	location_codes.location_code%TYPE);

type		col_collection is table of col_rec index by binary_integer;
col_recs	col_collection;
type		rec_type is table of programs%ROWTYPE index by binary_integer;

err_msg		long := null;

-- *********************************************************************************

function add_times (t1 integer, t2 integer) return integer is

this_date	date := trunc (sysdate);

begin

   this_date := common.add_time (this_date, t1);
   this_date := common.add_time (this_date, t2);

   return to_number (to_char (this_date, 'hh24mi'));

end add_times;

-- *********************************************************************************

function incr_time (itime number) return number is

str_date	varchar2(20);
dte_date	date;

begin

   if itime is null then
      return null;
   end if;

   str_date	:= to_char (sysdate, 'mmddyyy') || ' ' || to_char (itime, 'FM0000');
   dte_date	:= to_date (str_date, 'mmddyyyy hh24mi');
   dte_date	:= dte_date + common.program_interval$/1440;
   str_date	:= to_char (dte_date, 'hh24mi');

   return to_number (str_date);

end incr_time;

-- *********************************************************************************

function load_times return time_type is

cursor	C1 is
select	min (beg_time) beg_time, max (end_time) end_time
from	sessions;

cursor	C2 is
select	min (beg_time) beg_time, max (end_time) end_time
from	daily_events;

C2_rec		C1%ROWTYPE;
driver_rec	C1%ROWTYPE;

rec	time_type;

begin

   open C1;
   fetch C1 into driver_rec;
   close C1;

   open C2;
   fetch C2 into C2_rec;
   close C2;

   rec.beg_time := least(driver_rec.beg_time, nvl (C2_rec.beg_time, driver_rec.beg_time));
   rec.end_time := greatest(driver_rec.end_time, nvl (C2_rec.end_time, driver_rec.end_time));

   return rec;

end load_times;

-- *********************************************************************************

procedure load_locations is

cursor	driver (sdate date) is
select	location_code
from	location_codes
where	location_code	in
(	select	location_code
	from	sessions
	where	session_date	= sdate
	and	common.get_presentation_type2 (presentation_option_code) = common.oral$
)
order by location_descr;

location_codes	common.ident_arr;
itime		number;
sdate		date;
rec		time_type;
failSafe	integer;

begin

   time_rec := load_times;

   for i in 1..common.conf_len_days$ LOOP -- load every time in common.program_interval$ minute increments
      sdate := common.conf_start_monday$+i-1;

      open driver (sdate);
      fetch driver BULK COLLECT into location_codes;
      close driver;

      itime := time_rec.beg_time;
      failSafe := 0;
      LOOP
         failSafe := failSafe + 1;
         exit when failSafe > 100;
         exit when itime > rec.end_time;
         exit when itime is null;

         for j in 1..location_codes.COUNT LOOP
            insert into programs (day, beg_time, location_code) values (sdate, itime, location_codes(j));
         end loop;

         itime := incr_time (itime);
      end loop;
   end loop;

end load_locations;

-- *********************************************************************************

procedure load_events is

cursor	driver is
select	*
from	daily_events;

type	rec_type is table of daily_events%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	programs
      set	event_id	= recs(i).daily_event_id
      where	beg_time	= recs(i).beg_time;
   end loop;

end load_events;

-- *********************************************************************************

procedure validate_session (sid varchar2, btime integer) is

cursor	driver is
select	abstract_id, session_id, paper_id, start_time
from	abstract_sessions
where	session_id	= sid
and	start_time	< btime
order by session_id, session_seq;

begin

   for driver_rec in driver LOOP
      err_msg := err_msg || 'Paper ID "' ||
			htf.bold (nvl (driver_rec.paper_id, driver_rec.abstract_id)) ||
			'" in session ' || driver_rec.session_id || ' is scheduled to start at ' ||
			htf.bold (common.format_time (driver_rec.start_time)) ||
			', which is before the session start time of ' ||
			htf.bold (common.format_time (btime)) || '.' || htf.br;
   end loop;

end validate_session;

-- *********************************************************************************

procedure load_sessions is

cursor	driver is
select	*
from	sessions
where	common.get_presentation_type2 (presentation_option_code) = common.oral$;

begin

   for driver_rec in driver LOOP
      update	programs
      set	session_id	= driver_rec.session_id
      where	day		= driver_rec.session_date
      and	beg_time	= driver_rec.beg_time
      and	location_code	= driver_rec.location_code;

      validate_session (driver_rec.session_id, driver_rec.beg_time);
   end loop;

end load_sessions;

-- *********************************************************************************

procedure load_abstracts is

cursor	driver is
select	s.session_date, s.location_code, a.abstract_id, a.duration,
	a.start_time, a.session_id, a.paper_id
from	abstract_sessions	a,
	sessions		s
where	a.session_id		= s.session_id
and	a.start_time		is not null;

etime	integer;

begin

   for driver_rec in driver LOOP
      etime	:= add_times (driver_rec.start_time, driver_rec.duration);

      update	programs
      set	abstract_id	= driver_rec.abstract_id
      where	day		= driver_rec.session_date
      and	location_code	= driver_rec.location_code
      and	beg_time	>= driver_rec.start_time
      and	beg_time	< etime;

      if SQL%rowcount = 0 then
         err_msg := err_msg || 'No rows updated for ' ||
		nvl (driver_rec.paper_id, driver_rec.abstract_id) ||
		'. Time: ' || driver_rec.start_time || htf.br;
      end if;

   end loop;

end load_abstracts;

-- *********************************************************************************

procedure program_hdr is

cursor	C1 is
select	session_date, count (distinct location_code) icount
from	sessions
where	common.get_presentation_type2 (presentation_option_code) = common.oral$
group by
	session_date
order by session_date;

cursor	C2 (sdate date) is
select	distinct location_code
from	sessions
where	session_date	= sdate
and	common.get_presentation_type2 (presentation_option_code) = common.oral$
order by location_code;

begin

   htp.tableROwOpen ('CENTER', 'BOTTOM');
   htp.tableHeader ('Time', crowspan=>'2');
   for C1_rec in C1 LOOP
      htp.tableHeader (to_char (C1_rec.session_date, 'Day'), ccolspan=>C1_rec.icount);
   end loop;
   htp.tableRowClose;

   htp.tableROwOpen ('CENTER', 'BOTTOM');
   for C1_rec in C1 LOOP
      for C2_rec in C2 (C1_rec.session_date) LOOP
         htp.tableHeader (common.decode_value ('location_descr', 'location_codes',
				'location_code', C2_rec.location_code) );

         col_recs(col_recs.COUNT+1).day		:= C1_rec.session_date;
         col_recs(col_recs.COUNT+1).lcode	:= C2_rec.location_code;
      end loop;
   end loop;
   htp.tableRowClose;

end program_hdr;

-- *********************************************************************************

procedure show_session_info (sid varchar2) is

cursor	driver is
select	*
from	sessions
where	session_id = sid;

driver_rec	driver%ROWTYPE;

begin

   if sid is null then
      htp.tableData (common.space$);
   else
      open driver;
      fetch driver into driver_rec;
      close driver;

      htp.p ('<TH>');
      htp.p (driver_rec.session_id);
      htp.p ('-');
      htp.p (driver_rec.title);
      htp.p (common.space2$);

      if driver_rec.chair_profile_id is null then
         htp.italic ('Chair to be determined', cattributes=>common.red$);
      else
         htp.italic (common.get_user_name_affiliation (driver_rec.chair_profile_id, 'Y'));
      end if;

      htp.p ('</TH>');
   end if;

end show_session_info;

-- *********************************************************************************

function show_time_slot (itime integer, etime integer) return boolean is

-- Don't display the time slot if it is during a break or poster session.

cursor	driver is
select	1
from	programs
where	beg_time	= itime
and	event_id	is null;

dummy		integer;
rec_found	boolean;

begin

   if itime = etime then
      return false;
   end if;

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   return rec_found;

end show_time_slot;

-- *********************************************************************************

procedure show_abs_info (abs_id integer, rows_available integer) is

cursor	driver is
select	abs.*, common.get_paper_id (abs.abstract_id) paper_id,
	common.get_author_profile_id (abstract_id, common.speaker$) spk_profile_id
from	abstracts	abs
where	abstract_id	= abs_id;

driver_rec	driver%ROWTYPE;
buffer		varchar2(50) := '<TD>';

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   if rows_available > 1 then
      buffer := '<TD ROWSPAN="' || rows_available || '">';
   end if;

   htp.p (buffer);
   htp.p (driver_rec.paper_id);
   htp.p ('-');
   htp.p (driver_rec.title);
   htp.p (common.space2$);
   htp.italic (common.get_user_name_affiliation  (driver_rec.spk_profile_id));
   htp.p ('</TD>');

end show_abs_info;

-- *********************************************************************************

function time_has_session (recs rec_type) return boolean is

begin

   for i in 1..recs.COUNT LOOP
      if recs(i).session_id is not null then
         return true;
      end if;
   end loop;

   return false;

end time_has_session;

-- *********************************************************************************

function get_abs_row_span (btime integer, etime integer) return number is

-- rowspan is the number of time slots plus the empty slots for all sessions.

cursor	driver is
select	(
	select	count (distinct beg_time)
	from	sessions
	where	beg_time	> btime
	and	beg_time	< etime
	) 
	+
	(
	select	count (distinct beg_time)
	from	daily_events
	where	beg_time	> btime
	and	beg_time	< etime
	)
from	dual;

icount	integer;

bdate	date	:= common.add_time (trunc (sysdate), btime);
edate	date	:= common.add_time (trunc (sysdate), etime);
tdate	date	:= bdate;

begin

   open driver;
   fetch driver into icount;
   close driver;

   icount := nvl (icount, 0);
   loop
      exit when tdate >= edate;
      tdate	:= common.add_time (tdate, common.program_interval$);
      icount	:= icount + 1;
   end loop;

   return icount;

end get_abs_row_span;

-- *********************************************************************************

function get_abs_beg_time (abs_id integer) return integer is

cursor	driver is
select	min (beg_time)
from	programs
where	abstract_id = abs_id;

itime	integer;

begin

   open driver;
   fetch driver into itime;
   close driver;

   return itime;

end get_abs_beg_time;

-- *********************************************************************************

function skip_time_column (recs rec_type) return boolean is

begin

   for i in 1..recs.COUNT LOOP
      if recs(i).session_id is not null then
         for j in 1..i-1 LOOP
            if get_abs_beg_time (recs(j).abstract_id) < recs(i).beg_time then
               return true;
            end if;
         end loop;
      end if;
   end loop;

   return false;

end skip_time_column;

-- *********************************************************************************

function get_event_end_time (event_id integer) return integer is

begin

   return common.decode_value ('end_time', 'daily_events', 'daily_event_id', event_id);

end get_event_end_time;

-- *********************************************************************************

function get_event_title (event_id integer) return varchar2 is

begin

   return common.decode_value ('event', 'daily_events', 'daily_event_id', event_id);

end get_event_title;

-- *********************************************************************************

function get_abs_end_time (aid integer) return integer is

cursor	driver is
select	start_time, duration
from	abstract_sessions
where	abstract_id = aid;

driver_rec	driver%ROWTYPE;

begin

   open driver;
   fetch driver into driver_rec;
   close driver;

   return add_times (driver_rec.start_time, driver_rec.duration);

end get_abs_end_time;

-- *********************************************************************************

procedure show_program (btime number, etime number) is

cursor	driver (beg_time$ number) is
select	*
from	programs
where	beg_time = beg_time$
order by
	day, location_code;

rowspan		integer;
itime		integer := btime;
recs		rec_type;

failSafe	integer := 0;
abs_etime	integer;
event_etime	integer;

begin

   LOOP
      failSafe := failSafe + 1;
      exit when failSafe > 300;
      exit when itime > etime;

      recs.delete;
      open driver (itime);
      fetch driver BULK COLLECT into recs;
      close driver;

      if recs.COUNT > 0 then
         if recs(1).event_id is not null then

--	skip to the time slot after the break/event.
            for i in 1..recs.COUNT LOOP
               itime := greatest (itime, get_event_end_time (recs(i).event_id) );
            end loop;

            htp.tableRowOpen ('CENTER', 'MIDDLE');
            htp.tableHeader (	to_char (recs(1).beg_time, '0000') || ' - ' ||
				to_char (itime, '0000') , cnowrap=>'NOWRAP');
            htp.tableHeader (get_event_title (recs(1).event_id), ccolspan=>recs.COUNT);
            htp.tableRowClose;

         else

            if time_has_session (recs) then -- do not display abstracts (yet)
               htp.tableRowOpen ('LEFT', 'TOP');

               if NOT skip_time_column (recs) then
                  htp.tableData (common.space$);
               end if;

               for i in 1..recs.COUNT LOOP
                  show_session_info (recs(i).session_id);
               end loop;

               htp.tableRowClose;
            end if;

            htp.tableRowOpen ('LEFT', 'TOP');
            htp.tableData (to_char (itime, '0000'));

            for i in 1..recs.COUNT LOOP
               abs_etime := null;
               if recs(i).abstract_id is not null then
                  abs_etime := get_abs_end_time (recs(i).abstract_id);
               end if;

               if recs(i).abstract_id is null then
                  htp.tableData (common.space$);
               elsif recs(i).abstract_id is not null AND abs_etime = itime then
                  htp.tableData (common.space$);
               elsif get_abs_beg_time (recs(i).abstract_id) = itime then
                  rowspan := get_abs_row_span (recs(i).beg_time, abs_etime);
                  show_abs_info (recs(i).abstract_id, rowspan);
               elsif abs_etime is null then
                  htp.tableData (common.space$);
               end if;
            end loop;

            htp.tableRowClose;
            itime := incr_time (itime);
         end if;
      end if;

   end loop;

end show_program;

-- *********************************************************************************

procedure html is

begin

   if common.header (rpt_title$) then

      delete from programs;
      load_locations;
      load_events;
      load_sessions;
      load_abstracts;

-- **********

      if err_msg is null then
         htp.tableOpen ('BORDER');
         program_hdr;
         show_program (time_rec.beg_time, time_rec.end_time);
         htp.tableClose;
      else
         htp.header (3, 'ERROR - Unable to Produce the Conference Program', cattributes=>common.red$);
         htp.p (err_msg);
      end if;

      htp.para;
      common.footer;
   end if;

end html;

-- ***********************************************************************************************

end program;
/
sho error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>