set define off

create or replace package body rpt_stats as

type	abstract_type is record (
		total			integer,
		assignable		integer,
		assigned2editor		integer,
		has_editor_status	integer,
		assigned_no_status	integer,
		publishable		integer);

type	eStatus_type is record (
		rec			status_codes%ROWTYPE,
		total			integer,
		initial			integer,
		assignment_status	boolean default false);
type	eStatuses_type is table of eStatus_type index by binary_integer;

type	QA_type is record (
		ready_4QA		integer,
		publishable		integer,
		passed			integer,
		failed			integer);

type	stat_type is record (
		abstracts		abstract_type,
		eStatus			eStatuses_type,
		QA			QA_type);

red$	constant char(7)	:= '#FF0000';
green$	constant char(7)	:= '#00FF00';
yellow$	constant char(7)	:= '#FFFF00';
black$	constant char(7)	:= '#000000';
white$	constant char(7)	:= '#FFFFFF';

-- ***********************************************************************************

function load_abstract_stats return abstract_type is

cursor	C1 is
select	count(1), count(status_code)
from	abstracts;

cursor	C2 is
select	count(1), count(editor_id)
from	abstracts
where	assignable	>= common.file_weight_threshold$
or	editor_id	is not null;

cursor	C3 is
select	count(1)
from	abstracts	a
where	common.paper_OK_to_publish(abstract_id) = 'Yes';

cursor	C5 is
select	count(1)
from	abstracts
where	status_code	in
(	select	status_code
	from	log_codes
	where	log_code = common.paper_assigned$);

abs_rec	abstract_type;

begin

   open C1;
   fetch C1 into abs_rec.total, abs_rec.has_editor_status;
   close C1;

   open C2;
   fetch C2 into abs_rec.assignable, abs_rec.assigned2editor;
   close C2;

   open C3;
   fetch C3 into abs_rec.publishable;
   close C3;

   open C5;
   fetch C5 into abs_rec.assigned_no_status;
   close C5;

   return abs_rec;

end load_abstract_stats;

-- ***********************************************************************************

function load_editor_stats return eStatuses_type is

cursor	driver is
select	*
from	status_codes
where	status_code	not in
(	select	status_code
	from	log_codes
	where	log_code = common.paper_assigned$)
order by sort_order nulls last, status_descr;

cursor	C2 (code$ varchar2) is
select	count(1)
from	abstracts
where	status_code = code$;

cursor	C3 (code$ varchar2) is
select	count(abstract_id)
from	logs		l1
where	l1.status_code	= code$
and	l1.log_date	=
	(	select	min (log_date)
		from	logs	l2
		where	l2.abstract_id	= l1.abstract_id
		and	l2.status_code	not in
		(	select	status_code
			from	log_codes
			where	log_code = common.paper_assigned$
	)	);

erec	eStatuses_type;

begin

   for rec in driver LOOP
      erec(erec.COUNT+1).rec	:= rec;

      open C2 (rec.status_code);
      fetch C2 into erec(erec.COUNT).total;
      close C2;

      open C3 (rec.status_code);
      fetch C3 into erec(erec.COUNT).initial;
      close C3;
   end loop;

  return erec;

end load_editor_stats;

-- ***********************************************************************************

function load_QA_stats return QA_type is

cursor	driver is
select	l1.log_code, count(1) icount
from	logs		l1
where	l1.log_code	in (common.final_QA_passed$, common.final_QA_failed$)
and	l1.log_date	=
	(	select	max(log_date)
		from	logs	l2
		where	l2.abstract_id	= l1.abstract_id
		and	l2.log_code	in (common.final_QA_passed$, common.final_QA_failed$) )
group by l1.log_code;

cursor	C2 is
select	count(1)
from	abstracts
where	status_code in
	(	select	status_code
		from	status_codes
		where	publishable = 'Y');

QA_rec	QA_type;

begin

   QA_rec.passed := 0;
   QA_rec.failed := 0;

   for rec in driver LOOP
      if rec.log_code = common.final_QA_passed$ then
         QA_rec.passed := rec.icount;
      end if;

      if rec.log_code = common.final_QA_failed$ then
         QA_rec.failed := rec.icount;
      end if;
   end loop;

   open C2;
   fetch C2 into QA_rec.ready_4QA;
   close C2;

   return QA_rec;

end load_QA_stats;

-- ***********************************************************************************

function load_stats return stat_type is

rec	stat_type;

begin

   rec.abstracts 	:= load_abstract_stats;
   rec.eStatus		:= load_editor_stats;
   rec.QA		:= load_QA_stats;

   return rec;

end load_stats;

-- ***********************************************************************************
-- ***********************************************************************************

procedure show_row (	label varchar2,
			value integer,
			denom integer default 0,
			bgColor varchar2 default white$,
			fgColor	varchar2 default black$) is

style	varchar2(90)		:= null;
pct	varchar2(10)		:= common.space$;

begin

   style := 'STYLE="background-color: ' || bgColor || '; color:' || fgColor || '"';

   if denom <> 0 then
      pct := to_char (round (value*100/denom, 2), '990.00') || '%';
   end if;

   htp.tableRowOpen ('RIGHT', 'TOP');
   htp.tableHeader (label);
   htp.tableData (value, cattributes=>style);
   htp.tableData (pct);
   htp.tableRowClose;

end show_row;

-- ***********************************************************************************
-- ***********************************************************************************

procedure show_log_counts is

cursor	driver is
select	c.log_descr,
	count (l.abstract_id) icount
from	logs		l,
	log_codes	c
where	l.log_code	= c.log_code
and	c.log_descr	NOT like 'Email%'
and	c.log_code	<> 'AW'
group by c.log_descr
having count (l.abstract_id)	> 0
UNION
select	'Abstract Withdrawn', withdrawn_count +
		(	select count(abstract_id)
			from	logs
			where	log_code = 'AW')
from	control
where	withdrawn_count > 0
order by 2 desc, 1;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   htp.header (3, 'Activity Log');
   htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Count');
   htp.tableHeader ('Activity', 'LEFT');
   htp.tableRowClose;

   for i in 1..recs.COUNT loop
      htp.tableRowOpen ('LEFT', 'TOP');
      htp.tableData (to_char (recs(i).icount,  '999,999,999'), 'RIGHT');
      htp.tableData (recs(i).log_descr, cnowrap=>'NOWRAP');
      htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_log_counts;

-- ***********************************************************************************

procedure show_log_errors is

cursor	driver is
select	e.error_descr, count (l.abstract_id) icount
from	logs			l,
	log_errors		le,
	error_codes		e
where	le.error_code(+)	= e.error_code
and	l.log_id(+)		= le.log_id
group by e.error_descr
having count (l.abstract_id)	> 0
order by 2 desc, e.error_descr;

type	rec_type is table of driver%ROWTYPE index by binary_integer;
recs	rec_type;

total	integer := 0;
running	integer := 0;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT loop
      total := total + recs(i).icount;
   end loop;

   htp.header (3, 'Error Log');
   htp.tableOpen ('BORDER', cattributes=>'CELLPADDING="5"');

   htp.tableRowOpen ('RIGHT', 'BOTTOM');
   htp.tableHeader ('Count');
   htp.tableHeader ('Pct');
   htp.tableHeader ('Running');
   htp.tableHeader ('Error Type', 'LEFT');
   htp.tableRowClose;

   for i in 1..recs.COUNT loop
       running := running + recs(i).icount;

       htp.tableRowOpen ('LEFT', 'TOP');
       htp.tableData (to_char (recs(i).icount, '999,999,999'), 'RIGHT');

       if total = 0 then
          htp.tableData (common.space$);
       else
          htp.tableData (to_char (100*recs(i).icount/total, '990.00') || '%', 'RIGHT');
       end if;

       if running = 0 then
          htp.tableData (common.space$);
       else
          htp.tableData (to_char (100*running/total, '990.00') || '%', 'RIGHT');
       end if;

       htp.tableData (recs(i).error_descr);
       htp.tableRowClose;
   end loop;

   htp.tableClose;

end show_log_errors;

-- ***********************************************************************************
-- ***********************************************************************************

function get_assigned (platform varchar2, assigned_value integer) return integer is

cursor	driver is
select	count(distinct a.abstract_id)
from	files			f,
	presentation_file_codes	pfc,
	abstracts		a
where	a.editor_id		is not null
and	nvl(a.assignable,0)	= assigned_value
and	f.abstract_id		= a.abstract_id
and	f.platform_code		= platform
and	f.deleted		is null
and	pfc.file_type_code	= f.file_type_code
and	pfc.presentation_code	= a.presentation_code
and	pfc.weight		=
	(	select	max(pfc2.weight)
		from	presentation_file_codes	pfc2
		where	file_type_code		in
		(	select	file_type_code
			from	files		f2
			where	f2.abstract_id		= f.abstract_id
	)	);

icount	integer;

begin

   open driver;
   fetch driver into icount;
   close driver;

   return icount;

end get_assigned;

-- ***********************************************************************************

procedure show_abstract_stats (abs abstract_type) is

begin

   htp.tableOpen;
   htp.tableRowOpen;
   htp.p ('<TD>');
      htp.tableOpen ('BORDER');
      show_row ('Assignable',	abs.assignable, abs.total);
      show_row ('No Files',	abs.total-abs.assignable, abs.total, red$);
      show_row ('Unassigned',	abs.assignable-abs.assigned2editor, abs.assignable, yellow$);
      show_row ('Assigned',	abs.assigned2editor, abs.assignable, green$);
      htp.tableClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

end show_abstract_stats;

-- ***********************************************************************************

procedure show_status_stats (rec eStatuses_type, assigned_no_status integer) is

total	integer := assigned_no_status;

begin

   for i in 1..rec.COUNT LOOP
      total := total + rec(i).total;
   end loop;

   htp.tableOpen;
   htp.tableRowOpen;
   htp.p ('<TD>');
      htp.tableOpen ('BORDER');
      for i in 1..rec.COUNT loop
         show_row (rec(i).rec.status_descr, rec(i).total, total, '#' || rec(i).rec.edot_bg_color);
      end loop;
      show_row ('w/Editor', assigned_no_status, total, black$, white$);
      htp.tableClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

end show_status_stats;

-- ***********************************************************************************

procedure show_QA_stats (QA QA_type) is

begin

   htp.tableOpen;
   htp.tableRowOpen;

   htp.p ('<TD>');
      htp.tableOpen ('BORDER');
      show_row ('Final QA Failed', QA.failed, QA.ready_4QA, red$);
      show_row ('QA Remaining', QA.ready_4QA-QA.passed-QA.failed, QA.ready_4QA, yellow$);
      show_row ('Final QA Passed', QA.passed, QA.ready_4QA, green$);
      show_row ('Publishable', QA.publishable, QA.ready_4QA, white$);
      htp.tableClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

end show_QA_stats;

-- ***********************************************************************************

procedure show_initial_stats (rec eStatuses_type) is

total	integer := 0;

begin

   for i in 1..rec.COUNT LOOP
      total := total + rec(i).initial;
   end loop;

   htp.tableOpen;
   htp.tableRowOpen;
   htp.p ('<TD>');
      htp.tableOpen ('BORDER');
      for i in 1..rec.COUNT loop
         show_row (rec(i).rec.status_descr, rec(i).initial, total, '#' || rec(i).rec.edot_bg_color);
      end loop;
      htp.tableClose;
   htp.p ('</TD>');

   htp.tableRowClose;
   htp.tableClose;

end show_initial_stats;

-- ***********************************************************************************

procedure load_abstract_chart(abs abstract_type) is

begin

   charts.init_chart('Chart1', 'chart1_div');

   charts.add_option('Chart1', 'width',		'225');
   charts.add_option('Chart1', 'height',	'225');
   charts.add_option('Chart1', 'colors',	'["' || red$ || '","' || yellow$ || '","' || green$ || '"]');
   charts.add_option('Chart1', 'pieSliceText',	'"none"');
   charts.add_option('Chart1', 'is3D',		'true');
   charts.add_option('Chart1', 'legend',	'{position:"none"}');

   charts.add_column('Chart1', 'Status', 'Papers');
   charts.add_row('Chart1', 'No Files',		abs.total-abs.assignable);
   charts.add_row('Chart1', 'Unassigned',	abs.assignable-abs.assigned2editor);
   charts.add_row('Chart1', 'Assigned',		abs.assigned2editor);

end load_abstract_chart;

-- ***********************************************************************************

procedure load_status_stats (rec eStatuses_type, assigned_no_status integer) is

buffer	varchar2(4000);
sep	varchar2(12) := '","';

begin

   charts.init_chart('Chart2', 'chart2_div');

   charts.add_option('Chart2', 'width',		'225');
   charts.add_option('Chart2', 'height',	'225');
   charts.add_option('Chart2', 'pieSliceText',	'"none"');
   charts.add_option('Chart2', 'is3D',		'true');
   charts.add_option('Chart2', 'legend',	'{position:"none"}');

   charts.add_column('Chart2', 'Status', 'Papers');

   buffer := '["';
   for i in 1..rec.COUNT loop
      if i = rec.COUNT then
         sep := sep || black$ || '"]';
      end if;

      buffer := buffer || '#' || rec(i).rec.edot_bg_color || sep;
      charts.add_row('Chart2', rec(i).rec.status_descr, rec(i).total);
   end loop;

   charts.add_row('Chart2', 'w/Editor', assigned_no_status);
   charts.add_option('Chart2', 'colors', buffer);

end load_status_stats;

-- ***********************************************************************************

procedure load_QA_stats (QA QA_type) is

begin

   charts.init_chart('Chart3', 'chart3_div');

   charts.add_option('Chart3', 'width',		'225');
   charts.add_option('Chart3', 'height',	'225');
   charts.add_option('Chart3', 'colors',	'["' || red$ || '","' || yellow$ || '","' || green$ || '","' || white$ || '"]');
   charts.add_option('Chart3', 'pieSliceText',	'"none"');
   charts.add_option('Chart3', 'is3D',		'true');
   charts.add_option('Chart3', 'legend',	'{position:"none"}');

   charts.add_column('Chart3', 'Status', 'Papers');

   charts.add_row('Chart3', 'Final QA Failed',	QA.failed);
   charts.add_row('Chart3', 'QA Remaining',	QA.ready_4QA-QA.passed-QA.failed);
   charts.add_row('Chart3', 'Final QA Passed',	QA.passed);
   charts.add_row('Chart3', 'Publishable',	QA.publishable);

end load_QA_stats;

-- ***********************************************************************************

procedure load_initial_stats (rec eStatuses_type) is

buffer	varchar2(4000);
sep	varchar2(3) := '","';

begin

   charts.init_chart('Chart4', 'chart4_div');

   charts.add_option('Chart4', 'width',		'225');
   charts.add_option('Chart4', 'height',	'225');
   charts.add_option('Chart4', 'pieSliceText',	'"none"');
   charts.add_option('Chart4', 'is3D',		'true');
   charts.add_option('Chart4', 'legend',	'{position:"none"}');

   charts.add_column('Chart4', 'Status', 'Papers');

   buffer := '["';
   for i in 1..rec.COUNT loop
      if i = rec.COUNT then
         sep := '"]';
      end if;

      buffer := buffer || '#' || rec(i).rec.edot_bg_color || sep;
      charts.add_row('Chart4', rec(i).rec.status_descr, rec(i).initial);
   end loop;

   charts.add_option('Chart4', 'colors', buffer);

end load_initial_stats;

-- ***********************************************************************************

procedure html is

rec	stat_type;

begin

   rec := load_stats;
   load_abstract_chart(rec.abstracts);
   load_status_stats(rec.eStatus, rec.abstracts.assigned_no_status);
   load_QA_stats(rec.QA);
   load_initial_stats(rec.eStatus);

   if charts.header ('Statistics') then
      rec.QA.publishable := rec.abstracts.publishable;

      htp.tableOpen;

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.p ('<TD>');
         htp.bold ('Contributions (' || rec.abstracts.total || ')');
         show_abstract_stats(rec.abstracts);
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.bold ('Editor');
         show_status_stats(rec.eStatus, rec.abstracts.assigned_no_status);
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.bold ('Final QA');
         show_QA_stats(rec.QA);
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.bold ('Initial');
         show_initial_stats(rec.eStatus);
      htp.p ('<TD>');
      htp.tableRowClose;

--

      htp.tableRowOpen ('CENTER', 'TOP');
      htp.p ('<TD>');
         htp.p('<div id="chart1_div"></div>');
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.p('<div id="chart2_div"></div>');
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.p('<div id="chart3_div"></div>');
      htp.p ('<TD>');

      htp.p ('<TD>');
         htp.p('<div id="chart4_div"></div>');
      htp.p ('<TD>');
      htp.tableRowClose;

      htp.tableClose;

-- **********

      htp.tableOpen;
         htp.tableRowOpen ('LEFT', 'TOP');
            htp.p ('<TD>');
               rpt_papers_by_platform.load_data;
               rpt_papers_by_platform.display_data (false);
            htp.p ('</TD>');
         htp.tableRowClose;
      htp.tableClose;

      htp.para;
      htp.hr;

      htp.tableOpen;
      htp.tableRowOpen ('LEFT', 'TOP');

      htp.p ('<TD>');
         htp.para;
         show_log_counts;
      htp.p ('</TD>');

      htp.tableData (common.space2$);
      htp.tableData (common.space2$);

      htp.p ('<TD>');
         htp.para;
         show_log_errors;
      htp.p ('</TD>');

      htp.tableRowClose;
      htp.tableClose;

      common.footer;
   end if;

end html;

-- ***********************************************************************************

end rpt_stats;
/
show errors

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
