set define off

create or replace package body sync as

rpt_title$	constant varchar2(50)	:=  'Sync to JACoW Database';
process_name$	varchar2(500)		:= common.conf_title$ || ' JACoW SYNC';
last_sync_id	integer;
max_sync_id	integer			:= 0;

type	sync_data_type is record (
	control			repo_sync_data%ROWTYPE,
	person			repo_sync_persons%ROWTYPE,
	profile			repo_sync_profiles%ROWTYPE,
	address			repo_sync_addresses%ROWTYPE,
	affiliation		repo_sync_affiliations%ROWTYPE,
	merge_person		repo_sync_merge_persons%ROWTYPE,
	merge_profile		repo_sync_merge_profiles%ROWTYPE,
	membership		repo_sync_person_memberships%ROWTYPE,
	mailing_list		repo_sync_mailing_lists%ROWTYPE);

type	sync_data_recs is table of sync_data_type index by binary_integer;
recs	sync_data_recs;

rebuild_ADDRESS_CTX	boolean := false;
rebuild_AFFILIATION_CTX	boolean	:= false;

duplicate_key	exception;
integrity_FK	exception;
no_parent	exception;

pragma		exception_init (duplicate_key, -1);
pragma		exception_init (integrity_FK, -2292);
pragma		exception_init (no_parent, -2291);

-- **************************************************************************************************
-- The following routines run on the main repository only.
-- **************************************************************************************************

procedure purge is

cursor	driver is
select	max (sync_id)
from	sync_data
where	sync_id		<= (select min (last_sync_id) from conf_conferences where last_sync_id	is not null)
and	timestamp	< trunc(sysdate) - 14;

last_ID	integer;

begin

   open driver;
   fetch driver into last_ID;
   close driver;

   if last_ID is not null then
      delete from sync_addresses		where sync_id <= last_ID;
      delete from sync_affiliations		where sync_id <= last_ID;
      delete from sync_mailing_lists		where sync_id <= last_ID;
      delete from sync_merge_persons		where sync_id <= last_ID;
      delete from sync_merge_profiles		where sync_id <= last_ID;
      delete from sync_person_memberships	where sync_id <= last_ID;
      delete from sync_persons			where sync_id <= last_ID;
      delete from sync_profiles			where sync_id <= last_ID;
      delete from sync_data			where sync_id <= last_ID;
   end if;

end purge;

-- **************************************************************************************************

procedure initialize (rec in out conf_conferences%ROWTYPE) is

begin

   if common.repo_db$ is null then
      raise_application_error (-20004, 'The initialization routine must run on the repository only.');
   end if;

   if rec.conference_id is null then
      select conference_id_seq.nextval into rec.conference_id from dual;
      select max (sync_id) into rec.last_sync_id from repo_sync_data where sync_id is not null;
      insert into conf_conferences values rec;
   else
      update	conf_conferences
      set	last_sync_id	= (select max (sync_id) from sync_data where sync_id is not null)
      where	conference_id	= rec.conference_id;
   end if;

end initialize;

-- **************************************************************************************************

procedure set_last_sync_ID (conf_id integer, last_id integer) is

i integer;

begin

   if common.repo_db$ is null then
      raise_application_error (-20004, 'The initialization routine must run on the repository only.');
   end if;

   update conf_conferences
   set	  last_sync_id	= last_id
   where  conference_id	= conf_id;

end set_last_sync_ID;

-- **************************************************************************************************

function get_last_sync_ID (conf_id integer) return integer is

cursor	driver is
select	last_sync_id, repatriated
from	conf_conferences
where	conference_id = conf_id;

repatriated	date;

begin

   if common.repo_db$ is null then
      raise_application_error (-20004, 'The initialization routine must run on the repository only.');
   end if;

   open driver;
   fetch driver into last_sync_id, repatriated;
   close driver;

   if last_sync_id is null then
      raise_application_error (-20004, 'Last Sync ID not found.');
   end if;

   if repatriated is NOT null then
      raise_application_error (-20004, 'This conference was repatriated. SYNC is disabled.');
   end if;

   return last_sync_id;

end get_last_sync_ID;

-- **************************************************************************************************
-- The remainder of the routines run on the local database only.
-- **************************************************************************************************

function get_conference_id return integer is

conf_id		integer;

begin

   select conference_id into conf_id from control;
   if conf_id is null then
      raise_application_error (-20004, 'No conference ID found in main repository.');
   end if;

   return conf_id;

end get_conference_id;

-- **************************************************************************************************

function get_next_run_time return date is

-- unavailable_times are time frames where the main repository is unavailable

cursor	driver is
select	max (end_date)
from	unavailable_times
where	common.get_UTC_time + (common.minutes_between_sync$/1440)
		between beg_date and end_date;

end_time	date	:= sysdate + (common.minutes_between_sync$/1440);
next_avail	date	:= null;

begin

   open driver;
   fetch driver into next_avail;
   close driver;

   if next_avail is null then
      next_avail := end_time;
   else
      next_avail := common.local_time_stamp (next_avail);
   end if;

   return greatest (end_time, next_avail);

end get_next_run_time;

-- **************************************************************************************************

function check_DST (run_time date) return date is

cursor	driver is
select	*
from	DST_codes
where	database_name	= SYS_CONTEXT('USERENV', 'INSTANCE_NAME')
and	month		= to_number(to_char(run_time,'mm'));

rec_found	boolean;
rec		driver%ROWTYPE;
DST_date	date;

begin

   open driver;
   fetch driver into rec;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      return run_time;
   end if;

   if rec.week_number is null then -- Last Sunday of the Month
      DST_date := last_day (trunc(run_time));
      LOOP
         exit when to_number(to_char(DST_date,'d')) = 7; -- Sunday
         DST_date := DST_date - 1;
      end loop;

   else
      DST_date := last_day (trunc(run_time)) + 1;
      DST_date := add_months (DST_date, -1); -- First day of month for run_time

      LOOP
         if to_number(to_char(DST_date,'d')) = 1 then
            rec.week_number := rec.week_number - 1;
         end if;

         exit when rec.week_number = 0;
         DST_date := DST_date + 1;
      end loop;

   end if;

   if trunc(run_time) <> DST_date then
      return run_time;
   end if;

-- Don't allow a run between 0200 and 0259 during a DST adjustment
   if run_time between DST_date + 120/1440 and DST_date + 180/1440 then
      return DST_date + 180/1440; -- Next run at 0300
   end if;

   return run_time;

end check_DST;

-- **************************************************************************************************

function get_next_time return date is

next_run_time	date;

begin

   next_run_time := get_next_run_time;
   next_run_time := check_DST (next_run_time);

   return next_run_time;

end get_next_time;

-- **************************************************************************************************

procedure load_unavailable_times is

cursor	driver is
select	*
from	repo_unavailable_times
where	beg_date >= common.get_UTC_time;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	unavailable_times
      set	beg_date		= recs(i).beg_date,
		end_date		= recs(i).end_date,
		reason			= recs(i).reason
      where	unavailable_time_id	= recs(i).unavailable_time_id;

      if SQL%ROWCOUNT = 0 then
         insert into unavailable_times 
		(unavailable_time_id, beg_date, end_date, reason)
         values (recs(i).unavailable_time_id, recs(i).beg_date,
		recs(i).end_date, recs(i).reason);
      end if;
   end loop;

end load_unavailable_times;

-- **************************************************************************************************

procedure load_conference_codes is

cursor	driver is
select	*
from	REPO_conference_codes;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	conference_codes
      set	conference_descr	= recs(i).conference_descr
      where	conference_code		= recs(i).conference_code;

      if SQL%ROWCOUNT = 0 then
         insert into conference_codes (conference_code, conference_descr)
         values (recs(i).conference_code, recs(i).conference_descr);
      end if;
   end loop;

end load_conference_codes;

-- **************************************************************************************************

procedure load_country_codes is

cursor	driver is
select	*
from	REPO_country_codes;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	country_codes
      set	country_abbrev		= recs(i).country_abbrev,
		country_descr		= recs(i).country_descr,
		region_code		= recs(i).region_code
      where	country_code		= recs(i).country_code;

      if SQL%rowcount = 0 then
         insert into country_codes (country_code, country_abbrev, country_descr, region_code)
         values (recs(i).country_code, recs(i).country_abbrev, recs(i).country_descr, recs(i).region_code);
      end if;
   end loop;

end load_country_codes;

-- **************************************************************************************************

procedure load_region_codes is

cursor	driver is
select	*
from	REPO_region_codes;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	region_codes
      set	region_descr		= recs(i).region_descr
      where	region_code		= recs(i).region_code;

      if SQL%rowcount = 0 then
         insert into region_codes (region_code, region_descr)
         values (recs(i).region_code, recs(i).region_descr);
      end if;
   end loop;

end load_region_codes;

-- **************************************************************************************************

procedure load_affil_type_codes is

cursor	driver is
select	*
from	REPO_affil_type_codes;

type	rec_type is table of driver%ROWTYPE;
recs	rec_type;

begin

   open driver;
   fetch driver BULK COLLECT into recs;
   close driver;

   for i in 1..recs.COUNT LOOP
      update	affil_type_codes
      set	affil_type_descr = recs(i).affil_type_descr
      where	affil_type_code	 = recs(i).affil_type_code;

      if SQL%rowcount = 0 then
         insert into affil_type_codes (affil_type_code, affil_type_descr)
         values (recs(i).affil_type_code, recs(i).affil_type_descr);
      end if;
   end loop;

end load_affil_type_codes;

-- **************************************************************************************************
-- **************************************************************************************************

procedure load_sync_data is

cursor	driver is
select	*
from	repo_sync_data
where	sync_id	> last_sync_id;

type	lrecs_type is table of repo_sync_data%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).control := lrecs(i);
   end loop;

   if lrecs.COUNT > 0 then
      max_sync_id := recs(recs.LAST).control.sync_id;
   end if;

end load_sync_data;

-- **************************************************************************************************

procedure load_sync_persons is

cursor	driver is
select	*
from	repo_sync_persons
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_persons%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).person := lrecs(i);
   end loop;

end load_sync_persons;

-- **************************************************************************************************

procedure load_sync_merge_persons is

cursor	driver is
select	*
from	repo_sync_merge_persons
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_merge_persons%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).merge_person := lrecs(i);
   end loop;


end load_sync_merge_persons;

-- **************************************************************************************************

procedure load_sync_merge_profiles is

cursor	driver is
select	*
from	repo_sync_merge_profiles
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_merge_profiles%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).merge_profile := lrecs(i);
   end loop;


end load_sync_merge_profiles;

-- **************************************************************************************************

procedure load_sync_profiles is

cursor	driver is
select	*
from	repo_sync_profiles
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_profiles%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).profile := lrecs(i);
   end loop;

end load_sync_profiles;

-- **************************************************************************************************

procedure load_sync_addresses is

cursor	driver is
select	*
from	repo_sync_addresses
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_addresses%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).address := lrecs(i);
   end loop;

end load_sync_addresses;

-- **************************************************************************************************

procedure load_sync_affiliations is

cursor	driver is
select	*
from	repo_sync_affiliations
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_affiliations%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).affiliation := lrecs(i);
   end loop;

end load_sync_affiliations;

-- **************************************************************************************************

procedure load_sync_memberships is

cursor	driver is
select	*
from	repo_sync_person_memberships
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_person_memberships%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).membership := lrecs(i);
   end loop;

end load_sync_memberships;

-- **************************************************************************************************

procedure load_sync_mailing_lists is

cursor	driver is
select	*
from	repo_sync_mailing_lists
where	sync_id	between last_sync_id and max_sync_id;

type	lrecs_type is table of repo_sync_mailing_lists%ROWTYPE index by binary_integer;
lrecs	lrecs_type;

begin

   open driver;
   fetch driver BULK COLLECT into lrecs;
   close driver;

   for i in 1..lrecs.COUNT loop
      recs(lrecs(i).sync_id).mailing_list := lrecs(i);
   end loop;

end load_sync_mailing_lists;

-- **************************************************************************************************

procedure load_missing_records is

begin

   insert into affiliations
	select	*
	from	repo_affiliations
	where	affiliation_id not in (select affiliation_id from affiliations);

   insert into addresses
	select	*
	from	repo_addresses
	where	address_id not in (select address_id from addresses);

   insert into profiles
	select	*
	from	repo_profiles	rp
	where	rp.profile_id not in (select profile_id from profiles)
	and	not exists
		(	select	1
			from	profiles	p
			where	p.person_id	= rp.person_id
			and	p.address_id	= rp.address_id
		);

   insert into persons
	select	*
	from	repo_persons
	where	person_id not in (select person_id from persons);

end load_missing_records;

-- **************************************************************************************************
-- **************************************************************************************************

procedure process_person (idx integer) is

begin

   if recs(idx).control.action = 'INSERT' then

     insert into persons (person_id, user_id, password, fname, mname, lname,
		pname, initials, title, country_code, tz_name, date_format_code,
		authorization_given, admin_comments, gender)
      values (recs(idx).person.person_id, recs(idx).person.user_id, recs(idx).person.password,
		recs(idx).person.fname, recs(idx).person.mname, recs(idx).person.lname,
		recs(idx).person.pname, recs(idx).person.initials, recs(idx).person.title,
		recs(idx).person.country_code, recs(idx).person.tz_name, recs(idx).person.date_format_code,
		recs(idx).person.authorization_given, recs(idx).person.admin_comments,
		recs(idx).person.gender);

   elsif recs(idx).control.action = 'UPDATE' then
      update	persons
      set	user_id			= recs(idx).person.user_id,
		password		= recs(idx).person.password,
		fname			= recs(idx).person.fname,
		mname			= recs(idx).person.mname,
		lname			= recs(idx).person.lname,
		pname			= recs(idx).person.pname,
		initials		= recs(idx).person.initials,
		title			= recs(idx).person.title,
		country_code		= recs(idx).person.country_code,
		tz_name			= recs(idx).person.tz_name,
		date_format_code	= recs(idx).person.date_format_code,
		authorization_given	= recs(idx).person.authorization_given,
		gender			= recs(idx).person.gender
      where	person_id		= recs(idx).person.person_id;

   elsif recs(idx).control.action = 'DELETE' then
      delete from activities		where person_id = recs(idx).person.person_id;
      delete from person_memberships	where person_id = recs(idx).person.person_id;
      delete from mailing_lists		where person_id = recs(idx).person.person_id;
      delete from user_functions	where person_id = recs(idx).person.person_id;
      delete from logs			where person_id = recs(idx).person.person_id;
      delete from favorite_authors	where person_id = recs(idx).person.person_id;
      delete from person_schedules	where person_id = recs(idx).person.person_id;
      delete from mail_blats		where person_id = recs(idx).person.person_id;
      delete from mail_sent_blats	where person_id = recs(idx).person.person_id;
      delete from mail_blats_errors	where person_id = recs(idx).person.person_id;
      delete from persons		where person_id = recs(idx).person.person_id;

   elsif recs(idx).control.action = 'MERGE' then
      profile.merge3 (recs(idx).merge_person.old_person_id, recs(idx).merge_person.new_person_id);

   end if;

exception
   when duplicate_key	then null;
   when integrity_FK	then null;

end process_person;

-- **************************************************************************************************

procedure merge_profile (old_profile_id integer, new_profile_id integer) is

begin

   update authors		set profile_id = new_profile_id where profile_id = old_profile_id;
   update conf_roles		set profile_id = new_profile_id where profile_id = old_profile_id;
   update deleted_profiles	set profile_id = new_profile_id where profile_id = old_profile_id;
   update favorite_authors	set profile_id = new_profile_id where profile_id = old_profile_id;
   update jn_authors		set profile_id = new_profile_id where profile_id = old_profile_id;
   update sessions		set chair_profile_id = new_profile_id where chair_profile_id = old_profile_id;

end merge_profile;

-- **************************************************************************************************

procedure check_person (pid integer) is

cursor	driver is
select	1
from	persons
where	person_id = pid;

rec_found	boolean;
dummy		integer;

begin

   open driver;
   fetch driver into dummy;
   rec_found := driver%FOUND;
   close driver;

   if NOT rec_found then
      insert into persons select * from repo_persons where person_id = pid;
   end if;

end check_person;

-- **************************************************************************************************

procedure process_profile (idx integer) is

begin

   if recs(idx).control.action = 'INSERT' then
      check_person (recs(idx).profile.person_id);

      insert into profiles (	profile_id, person_id, address_id, email_address,
				mail_stop, po_box, phone_number, fax_number, sort_order,
				current_address, deleted)
      values (recs(idx).profile.profile_id, recs(idx).profile.person_id, recs(idx).profile.address_id,
		recs(idx).profile.email_address, recs(idx).profile.mail_stop, recs(idx).profile.po_box,
		recs(idx).profile.phone_number, recs(idx).profile.fax_number, recs(idx).profile.sort_order,
		recs(idx).profile.current_address, recs(idx).profile.deleted);

   elsif recs(idx).control.action = 'UPDATE' then
      update	profiles
      set	address_id	= recs(idx).profile.address_id,
		email_address	= recs(idx).profile.email_address,
		mail_stop	= recs(idx).profile.mail_stop,
		po_box		= recs(idx).profile.po_box,
		phone_number	= recs(idx).profile.phone_number,
		fax_number	= recs(idx).profile.fax_number,
		sort_order	= recs(idx).profile.sort_order,
		current_address	= recs(idx).profile.current_address,
		deleted		= recs(idx).profile.deleted
      where	profile_id	= recs(idx).profile.profile_id;

   elsif recs(idx).control.action = 'DELETE' then
      delete from profiles where profile_id = recs(idx).profile.profile_id;

   elsif recs(idx).control.action = 'MERGE' then
      merge_profile (recs(idx).merge_profile.old_profile_id, recs(idx).merge_profile.new_profile_id);

   end if;

exception
   when duplicate_key	then null;
   when integrity_FK	then null;

end process_profile;

-- **************************************************************************************************

procedure process_address (idx integer) is

addrs	common.ident_arr;

begin

   rebuild_ADDRESS_CTX := true;

   if recs(idx).control.action = 'INSERT' then
      insert into addresses (address_id, address1, address2, address3, po_box,
		town, postal_code, zip_code, country_code, affiliation_id)
      values (recs(idx).address.address_id, recs(idx).address.address1, recs(idx).address.address2,
		recs(idx).address.address3, recs(idx).address.po_box, recs(idx).address.town,
		recs(idx).address.postal_code, recs(idx).address.zip_code,
		recs(idx).address.country_code, recs(idx).address.affiliation_id);

   elsif recs(idx).control.action = 'UPDATE' then
      update	addresses
      set	address1	= recs(idx).address.address1,
		address2	= recs(idx).address.address2,
		address3	= recs(idx).address.address3,
		po_box		= recs(idx).address.po_box,
		town		= recs(idx).address.town,
		postal_code	= recs(idx).address.postal_code,
		zip_code	= recs(idx).address.zip_code,
		country_code	= recs(idx).address.country_code,
		affiliation_id	= recs(idx).address.affiliation_id
      where	address_id	= recs(idx).address.address_id;

   elsif recs(idx).control.action = 'DELETE' then
      delete from addresses where address_id = recs(idx).address.address_id;

   end if;

exception
   when duplicate_key	then null;
   when integrity_FK	then null;

end process_address;

-- **************************************************************************************************

procedure process_affiliation (idx integer) is

begin

   rebuild_AFFILIATION_CTX := true;

   if recs(idx).control.action = 'INSERT' then
      insert into affiliations (affiliation_id, affil_abbrev, name1, name2,
				department, web_site, affil_type_code)
      values (recs(idx).affiliation.affiliation_id, recs(idx).affiliation.affil_abbrev,
		recs(idx).affiliation.name1, recs(idx).affiliation.name2,
		recs(idx).affiliation.department, recs(idx).affiliation.web_site,
		recs(idx).affiliation.affil_type_code);

   elsif recs(idx).control.action = 'UPDATE' then
      update	affiliations
      set	affil_abbrev	= recs(idx).affiliation.affil_abbrev,
		name1		= recs(idx).affiliation.name1,
		name2		= recs(idx).affiliation.name2,
		department	= recs(idx).affiliation.department,
		web_site	= recs(idx).affiliation.web_site,
		affil_type_code	= recs(idx).affiliation.affil_type_code
      where	affiliation_id	= recs(idx).affiliation.affiliation_id;

   elsif recs(idx).control.action = 'DELETE' then
      delete from affiliations where affiliation_id = recs(idx).affiliation.affiliation_id;

   end if;

exception
   when duplicate_key	then null;
   when integrity_FK	then null;

end process_affiliation;

-- **************************************************************************************************

procedure process_membership (idx integer) is

-- Updates are a result of a merged person.

begin

   if recs(idx).control.action = 'INSERT' then
      insert into person_memberships (person_id, membership_code, membership_number)
      values 	(recs(idx).membership.person_id,
		recs(idx).membership.membership_code,
		recs(idx).membership.membership_number);

   elsif recs(idx).control.action = 'DELETE' then
      delete from person_memberships
      where person_id		= recs(idx).membership.person_id
      and   membership_code	= recs(idx).membership.membership_code;

   end if;

exception
   when duplicate_key	then null;
   when no_parent	then null;

end process_membership;

-- **************************************************************************************************

procedure process_mailing_list (idx integer) is

-- Updates are a result of a merged person.

begin

   if recs(idx).control.action = 'INSERT' then
      insert into mailing_lists (person_id, conference_code)
      values (recs(idx).mailing_list.person_id, recs(idx).mailing_list.conference_code);

   elsif recs(idx).control.action = 'DELETE' then
      delete from mailing_lists
      where person_id		= recs(idx).mailing_list.person_id
      and   conference_code	= recs(idx).mailing_list.conference_code;

   end if;

exception
   when duplicate_key	then null;
   when no_parent	then null;

end process_mailing_list;

-- **************************************************************************************************

procedure process_sync_data is

--I INTEGER;
--J INTEGER := 0;

idx	integer;

begin

   idx := recs.FIRST;
   LOOP
      exit when idx is null;

      if recs(idx).control.table_name = 'PERSONS' then
         process_person (idx);

      elsif recs(idx).control.table_name = 'PROFILES' then
         process_profile (idx);

      elsif recs(idx).control.table_name = 'ADDRESSES' then
         process_address (idx);

      elsif recs(idx).control.table_name = 'AFFILIATIONS' then
         process_affiliation (idx);

      elsif recs(idx).control.table_name = 'PERSON_MEMBERSHIPS' then
         process_membership (idx);

      elsif recs(idx).control.table_name = 'MAILING_LISTS' then
         process_mailing_list (idx);

      end if;

      idx := recs.NEXT(idx);
   end loop;

end process_sync_data;

-- **************************************************************************************************

procedure one_person (pid integer) is

cursor	driver is
select	job
from	user_jobs
where	what = 'sync.repo;';

job_id	integer;

begin

   open driver;
   fetch driver into job_id;
   close driver;

   if job_id is not null then -- run the job in the background
      dbms_job.run (job_id);
   end if;

end one_person;

-- **************************************************************************************************

procedure repo is

conf_id		constant integer := get_conference_id;
last_id		number;

begin

   if common.process_is_running (process_name$) then
      dbms_output.put_line ('Already Running ' || process_name$);
   elsif common.repo_db$ is NOT null then
      raise_application_error (-20004, 'This IS the repository database or a standalone DB. No need to SYNC.');
   else
      common.set_process_name (process_name$);

      last_sync_id := repo_sync.get_last_sync_ID (conf_id);

      load_affil_type_codes;
      load_country_codes;
      load_conference_codes;
      load_unavailable_times;

      load_missing_records;

      load_sync_data;
      load_sync_persons;
      load_sync_profiles;
      load_sync_addresses;
      load_sync_affiliations;
      load_sync_memberships;
      load_sync_mailing_lists;
      load_sync_merge_persons;
      load_sync_merge_profiles;

      process_sync_data;

      if rebuild_ADDRESS_CTX then
         common.rebuild_CTX ('ADDRESS_CTX');
      end if;

      if rebuild_AFFILIATION_CTX then
         common.rebuild_CTX ('AFFILIATION_CTX');
      end if;

      if recs.COUNT > 0 then
         last_id := recs(recs.LAST).control.sync_id;
         repo_sync.set_last_sync_ID (conf_id, last_id);
         update	control set last_sync_id = last_id;
      end if;
   end if;

   common.set_process_name ('');

end repo;

-- **************************************************************************************************

procedure push_registration_form_data is

cursor	driver is
select	conference_id
from	control
where	exists (select 1 from pages);

conf_id	number;

begin

   open driver;
   fetch driver into conf_id;
   close driver;

   if conf_id is not null then
      xml2.push_xml_data (conf_id, 'VAR_TYPE_CODES',		'select * from var_type_codes');
      xml2.push_xml_data (conf_id, 'SPMS_VAR_CODES',		'select * from spms_var_codes');
      xml2.push_xml_data (conf_id, 'PAYMENT_VENDOR_CODES',	'select * from payment_vendor_codes');
      xml2.push_xml_data (conf_id, 'PAYMENT_VENDOR_VARS',	'select * from payment_vendor_vars');
      xml2.push_xml_data (conf_id, 'DISPLAY_CODES',		'select * from display_codes');
      xml2.push_xml_data (conf_id, 'DATA_TYPE_CODES',		'select * from data_type_codes');
      xml2.push_xml_data (conf_id, 'REG_CAT_CODES',		'select * from reg_cat_codes');
      xml2.push_xml_data (conf_id, 'META_DATA_CODES',		'select * from meta_data_codes');
      xml2.push_xml_data (conf_id, 'POPLIST_ITEMS',		'select * from poplist_items');
      xml2.push_xml_data (conf_id, 'CONDITIONAL_REQUIREMENTS',	'select * from conditional_requirements');
      xml2.push_xml_data (conf_id, 'FEE_MATRIX',		'select * from fee_matrix');
      xml2.push_xml_data (conf_id, 'PAGES',			'select * from pages');
      xml2.push_xml_data (conf_id, 'PAGE_DATA',			'select * from page_data');
      xml2.push_xml_data (conf_id, 'PAYMENT_CODES',		'select * from payment_codes');
      xml2.push_xml_data (conf_id, 'REGIST_BYPASS_VARS',	'select * from regist_bypass_vars');
   end if;

end push_registration_form_data;

-- **************************************************************************************************

procedure push_XML_data is

cursor	driver is
select	conference_id
from	control;

conf_id	number;

begin

   open driver;
   fetch driver into conf_id;
   close driver;

   if conf_id is not null then
      push_registration_form_data;

      xml2.push_xml_data (conf_id, 'CLASSIFICATION_CODES',	'select * from classification_codes');
      xml2.push_xml_data (conf_id, 'SUB_CLASS_CODES',		'select * from sub_class_codes');
      xml2.push_xml_data (conf_id, 'ECLASS_CODES',		'select * from eclass_codes');
      xml2.push_xml_data (conf_id, 'USER_FUNCTIONS',		'select * from user_functions');
      xml2.push_xml_data (conf_id, 'FUNCTION_CODES',		'select * from function_codes');
      xml2.push_xml_data (conf_id, 'FUNCTION_PRIVILEGES',	'select * from function_privileges');
   end if;

end push_XML_data;

-- **************************************************************************************************

end sync;
/
show error

set define on

--	Copyright (C) 2004 Joint Accelerator Conference Web Site
--	<http://www.jacow.org> by Matthew Arena, Fermilab
--
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program; if not, write to the Free Software
--   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--   You can also get a copy of the license through the web at
--   <http://www.gnu.org/licenses/gpl.html>
